/*
 * $Copyright (c) 2015-2017, 2022 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its subsidiaries and/or its affiliates and/or their licensors.$
 * Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your License Agreement with Software AG
 */

#ifndef _SAG_INTERNAL_CPP_STDLIB_HPP_
#define _SAG_INTERNAL_CPP_STDLIB_HPP_

#include <type_traits>

namespace ampl {



// constants
using std::true_type;
using std::false_type;

// is_one_of selector
template<class T, class... P0toN> struct is_one_of;
template<class T>
struct is_one_of<T>: false_type {};
template<class T, class... P1toN>
struct is_one_of<T, T, P1toN...>: true_type {};
template<class T, class P0, class... P1toN>
struct is_one_of<T, P0, P1toN...>: is_one_of<T, P1toN...> {};

// is_same selector
template<typename A, typename B> struct is_same: false_type {};
template<typename A> struct is_same<A, A>: true_type {};

template<typename A, typename B>
using is_same_t = typename std::enable_if<is_same<A, B>::value, A>::type;
template<typename A, typename B>
using is_not_same_t = typename std::enable_if<!is_same<A, B>::value, A>::type;

// identity type
template<typename T>
struct type_is : true_type { using type = T; };

// without const
template<typename T>
using remove_const_t = typename std::remove_const<T>::type;

template<typename T>
using remove_ref_t = typename std::remove_reference<T>::type;

// template functions for turning iterating over template parameters to iterating over the static indexes of those parameters,
// for unpacking the list_t array which holds all the method parameters

template<size_t...> struct index_tuple {};

template<size_t N, size_t... S>
struct gens : gens<N-1, N-1, S...> {};
template<size_t... S>
struct gens<0, S...> {
		typedef index_tuple<S...> type;
};

template<size_t N>
struct make_index_tuple
{
		typedef typename gens<N>::type type;
};

} // ampl

#endif // _SAG_INTERNAL_CPP_STDLIB_HPP_
